/**************************************************************************//**
*   @file   main.c
*   @brief  System validation program
*   @author ACostina (adrian.costina@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 1353
******************************************************************************/

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <altera_avalon_pio_regs.h>
#include "system.h"
#include "adi.h"
#include "AD7688.h"

#define ADC_StartAcquisition AD7688_StartAcquisition
#define ADC_ReadStatus 		 AD7688_ReadStatus

/*****************************************************************************/
/************************ Constants Definitions ******************************/
/*****************************************************************************/
#define SAMPLE_NO 8*1024

/*****************************************************************************/
/************************ Variables Definitions ******************************/
/*****************************************************************************/
static unsigned int txData[SAMPLE_NO];				//array to store the data read from the ADC
static volatile uCProbeDataUnion ucProbe_Data[30]; 	//used to store variables for uC-Probe communication
static unsigned int size			= 0;            //used for the request of number of samples
volatile char  startAcquisition 	= 0;            //starts one acquisition cycle

volatile char* acquisitionInProgress         = (char*)&ucProbe_Data[0].chData;   //shows that data is being acquired from the evaluation board
volatile char* acquisitionComplete           = (char*)&ucProbe_Data[1].chData;
volatile char* transferInProgress            = (char*)&ucProbe_Data[2].chData;   //shows that data is being transfered from the CED1Z memory to the PC
volatile char* transferComplete              = (char*)&ucProbe_Data[3].chData;
volatile char* processingDataInProgress      = (char*)&ucProbe_Data[4].chData;   //shows that the TCL script is processing the acquired data
volatile char* processingDataComplete        = (char*)&ucProbe_Data[5].chData;
volatile char* movingToFileInProgres         = (char*)&ucProbe_Data[6].chData;   //shows that the final data is written in a CSV file
volatile char* movingToFileComplete          = (char*)&ucProbe_Data[7].chData;
volatile char* binaryOffset                  = (char*)&ucProbe_Data[8].chData;   //true if the ADC is configured to format data as binary offset
volatile char* twosComplement                = (char*)&ucProbe_Data[9].chData;   //true if the ADC is configured to format data as 2's complement
volatile char dataCoding                     = 1; //if 0 data is coded as binary offset,if non zero as 2's complement

/*****************************************************************************/
/************************ Functions Definitions ******************************/
/*****************************************************************************/

int main(void)
{

	volatile char* startAcquisitionTCL           = (char*)0x0001BFF0;
	volatile char* transferCompleteTCL           = (char*)0x0001BFF1;
	volatile char* processingDataTCL             = (char*)0x0001BFF2;
	volatile char* operationCompleteTCL          = (char*)0x0001BFF3;
	volatile char* binaryOffsetOrTwosComplement  = (char*)0x0001BFF4;
	volatile int*  dataArrayAddr				 = (int*) 0x0001BFF8;
    volatile int*  startAcqPointer        		 = (int*) 0x0001BFFC;

	*dataArrayAddr = (int)txData;

    //clear all LEDs
	ClearLed(LED_ALL);

    //signal the start of the program
    //printf ("Nios 2 processor started...\n");

    *startAcqPointer							 = &startAcquisition;

    while(1)
    {
    	*startAcquisitionTCL            = 0;
        *transferCompleteTCL            = 0;
        *processingDataTCL              = 0;
        *operationCompleteTCL           = 0;

        *twosComplement = dataCoding;
        *binaryOffset   = !dataCoding;

        *binaryOffsetOrTwosComplement = *binaryOffset;

        ToggleLed(LED_1);
        size = SAMPLE_NO;

        if(startAcquisition)
        {
        	startAcquisition            = 0;
            //printf ("Acquisition has been started...\n");

            //reset the PC transfer status variables
            *acquisitionInProgress      = 1;
            *acquisitionComplete        = 0;
            *transferInProgress         = 0;
            *transferComplete           = 0;
            *processingDataInProgress   = 0;
            *processingDataComplete     = 0;
            *movingToFileInProgres      = 0;
            *movingToFileComplete       = 0;

            ADC_StartAcquisition(size, txData);
            while(!(ADC_ReadStatus() & 0x00000001));

            if (ADC_ReadStatus() != 1)
            {
                //printf ("Error reading data: %x \n", ADC_ReadStatus());
            }

            *acquisitionComplete     = 1;
            *acquisitionInProgress   = 0;
            *transferInProgress      = 1;

            //signal the start of data transfer to the PC
            *startAcquisitionTCL = 1;

            //signal the end of data transfer to the PC
            while (*transferCompleteTCL == 0);
            *transferComplete           = 1;
            *transferInProgress         = 0;
            *processingDataInProgress   = 1;
            *startAcquisitionTCL        = 0;

            while (*processingDataTCL == 0);
            *processingDataComplete     = 1;
            *movingToFileInProgres      = 1;
            *processingDataInProgress   = 0;
            while (*operationCompleteTCL == 0);
            *movingToFileComplete       = 1;
            *movingToFileInProgres      = 0;
            //printf ("End of acquisition\n");
        }
    }
}
